@@ -1,5 +1,5 @@
-from sqlalchemy             import (create_session, relation, mapper, join, 
-                                    DynamicMetaData, class_mapper, util)
+from sqlalchemy             import create_session, relation, mapper, \
+join, DynamicMetaData, class_mapper, util
 from sqlalchemy             import and_, or_
 from sqlalchemy             import Table, Column, ForeignKey
 from sqlalchemy.ext.sessioncontext import SessionContext
@@ -232,4 +232,4 @@ def create_tables():
 
 def drop_tables():
     for metadata in ActiveMapperMeta.metadatas:
-        metadata.drop_all()
\ No newline at end of file
+        metadata.drop_all()
