@@ -8,7 +8,7 @@
 well as relationships.  also defines some MapperOptions that can be used with the
 properties."""
 
-from sqlalchemy import sql, schema, util, attributes, exceptions
+from sqlalchemy import sql, schema, util, attributes, exceptions, sql_util
 import sync
 import mapper
 import session as sessionlib
@@ -68,20 +68,22 @@ class DeferredColumnProperty(ColumnProperty):
         if not self.localparent.is_assigned(instance):
             return mapper.object_mapper(instance).props[self.key].setup_loader(instance)
         def lazyload():
-            session = sessionlib.object_session(instance)
-            if session is None:
-                return None
-            clause = sql.and_()
-            connection = session.connection(self.parent)
             try:
                 pk = self.parent.pks_by_table[self.columns[0].table]
             except KeyError:
                 pk = self.columns[0].table.primary_key
+
+            clause = sql.and_()
             for primary_key in pk:
                 attr = self.parent._getattrbycolumn(instance, primary_key)
                 if not attr:
                     return None
                 clause.clauses.append(primary_key == attr)
+
+            session = sessionlib.object_session(instance)
+            if session is None:
+                raise exceptions.InvalidRequestError("Parent instance %s is not bound to a Session; deferred load operation of attribute '%s' cannot proceed" % (instance.__class__, self.key))
+            connection = session.connection(self.parent)
             
             try:
                 if self.group is not None:
@@ -184,18 +186,25 @@ class PropertyLoader(mapper.MapperProperty):
         """template method for subclasses of PropertyLoader"""
         pass
     
+    def _get_target_class(self):
+        """return the target class of the relation, even if the property has not been initialized yet."""
+        if isinstance(self.argument, type):
+            return self.argument
+        else:
+            return self.argument.class_
+        
     def do_init(self, key, parent):
         import sqlalchemy.orm
         if isinstance(self.argument, type):
-            self.mapper = mapper.class_mapper(self.argument)
+            self.mapper = mapper.class_mapper(self.argument, compile=False)._do_compile()
         else:
-            self.mapper = self.argument.compile()
+            self.mapper = self.argument._do_compile()
 
-        self.mapper = self.mapper.get_select_mapper()
+        self.mapper = self.mapper.get_select_mapper()._do_compile()
             
         if self.association is not None:
             if isinstance(self.association, type):
-                self.association = mapper.class_mapper(self.association)
+                self.association = mapper.class_mapper(self.association, compile=False)
         
         self.target = self.mapper.mapped_table
 
@@ -274,7 +283,7 @@ class PropertyLoader(mapper.MapperProperty):
         elif len([c for c in self.foreignkey if self.parent.unjoined_table.corresponding_column(c, False) is not None]):
             return sync.MANYTOONE
         else:
-            raise exceptions.ArgumentError("Cant determine relation direction " + repr(self.foreignkey))
+            raise exceptions.ArgumentError("Cant determine relation direction '%s', for '%s' in mapper '%s' with primary join\n '%s'" %(repr(self.foreignkey), self.key, str(self.mapper), str(self.primaryjoin)))
             
     def _find_dependent(self):
         """searches through the primary join condition to determine which side
@@ -306,7 +315,7 @@ class PropertyLoader(mapper.MapperProperty):
 
     def register_dependencies(self, uowcommit):
         self._dependency_processor.register_dependencies(uowcommit)
-
+            
     def _compile_synchronizers(self):
         """assembles a list of 'synchronization rules', which are instructions on how to populate
         the objects on each side of a relationship.  This is done when a PropertyLoader is 
@@ -348,18 +357,19 @@ class LazyLoader(PropertyLoader):
         def lazyload():
             params = {}
             allparams = True
-            session = sessionlib.object_session(instance)
             #print "setting up loader, lazywhere", str(self.lazywhere), "binds", self.lazybinds
-            if session is not None:
-                for col, bind in self.lazybinds.iteritems():
-                    params[bind.key] = self.parent._getattrbycolumn(instance, col)
-                    if params[bind.key] is None:
-                        allparams = False
-                        break
-            else:
-                allparams = False
+            for col, bind in self.lazybinds.iteritems():
+                params[bind.key] = self.parent._getattrbycolumn(instance, col)
+                if params[bind.key] is None:
+                    allparams = False
+                    break
+                
             if not allparams:
                 return None
+
+            session = sessionlib.object_session(instance)
+            if session is None:
+                raise exceptions.InvalidRequestError("Parent instance %s is not bound to a Session; lazy load operation of attribute '%s' cannot proceed" % (instance.__class__, self.key))
                 
             # if we have a simple straight-primary key load, use mapper.get()
             # to possibly save a DB round trip
@@ -413,13 +423,13 @@ def create_lazy_clause(table, primaryjoin, secondaryjoin, foreignkey):
         if isinstance(binary.left, schema.Column) and isinstance(binary.right, schema.Column) and ((not circular and binary.left.table is table) or (circular and binary.right in foreignkey)):
             col = binary.left
             binary.left = binds.setdefault(binary.left,
-                    sql.BindParamClause(bind_label(), None, shortname = binary.left.name))
+                    sql.BindParamClause(bind_label(), None, shortname=binary.left.name, type=binary.right.type))
             reverse[binary.right] = binds[col]
 
         if isinstance(binary.right, schema.Column) and isinstance(binary.left, schema.Column) and ((not circular and binary.right.table is table) or (circular and binary.left in foreignkey)):
             col = binary.right
             binary.right = binds.setdefault(binary.right,
-                    sql.BindParamClause(bind_label(), None, shortname = binary.right.name))
+                    sql.BindParamClause(bind_label(), None, shortname=binary.right.name, type=binary.left.type))
             reverse[binary.left] = binds[col]
             
     lazywhere = primaryjoin.copy_container()
@@ -440,7 +450,7 @@ class EagerLoader(LazyLoader):
         self.eagertarget = self.target.alias()
         if self.secondary:
             self.eagersecondary = self.secondary.alias()
-            self.aliasizer = Aliasizer(self.target, self.secondary, aliases={
+            self.aliasizer = sql_util.Aliasizer(self.target, self.secondary, aliases={
                     self.target:self.eagertarget,
                     self.secondary:self.eagersecondary
                     })
@@ -451,7 +461,7 @@ class EagerLoader(LazyLoader):
             self.eagerprimary.accept_visitor(self.aliasizer)
             #print "JOINS:", str(self.eagerprimary), "|", str(self.eagersecondaryjoin)
         else:
-            self.aliasizer = Aliasizer(self.target, aliases={self.target:self.eagertarget})
+            self.aliasizer = sql_util.Aliasizer(self.target, aliases={self.target:self.eagertarget})
             self.eagerprimary = self.primaryjoin.copy_container()
             self.eagerprimary.accept_visitor(self.aliasizer)
         
@@ -495,7 +505,7 @@ class EagerLoader(LazyLoader):
                     p.do_init_subclass(prop.key, prop.parent, recursion_stack)
                     p._create_eager_chain(recursion_stack=recursion_stack)
                     p.eagerprimary = p.eagerprimary.copy_container()
-#                    aliasizer = Aliasizer(p.parent.mapped_table, aliases={p.parent.mapped_table:self.eagertarget})
+#                    aliasizer = sql_util.Aliasizer(p.parent.mapped_table, aliases={p.parent.mapped_table:self.eagertarget})
                     p.eagerprimary.accept_visitor(self.aliasizer)
                     #print "new eagertqarget", p.eagertarget.name, (p.secondary and p.secondary.name or "none"), p.parent.mapped_table.name
             finally:
@@ -684,6 +694,9 @@ class BackRef(object):
             mapper._compile_property(self.key, relation);
         else:
             # else set one of us as the "backreference"
+            parent = prop.parent.primary_mapper()
+            if parent.class_ is not mapper.props[self.key]._get_target_class():
+                raise exceptions.ArgumentError("Backrefs do not match:  backref '%s' expects to connect to %s, but found a backref already connected to %s" % (self.key, str(parent.class_), str(mapper.props[self.key].mapper.class_)))
             if not mapper.props[self.key].is_backref:
                 prop.is_backref=True
                 prop._dependency_processor.is_backref=True
@@ -730,34 +743,6 @@ class DeferredOption(GenericOption):
             prop = ColumnProperty(*oldprop.columns, **self.kwargs)
         mapper._compile_property(key, prop)
         
-class Aliasizer(sql.ClauseVisitor):
-    """converts a table instance within an expression to be an alias of that table."""
-    def __init__(self, *tables, **kwargs):
-        self.tables = {}
-        self.aliases = kwargs.get('aliases', {})
-        for t in tables:
-            self.tables[t] = t
-            if not self.aliases.has_key(t):
-                self.aliases[t] = sql.alias(t)
-            if isinstance(t, sql.Join):
-                for t2 in t.columns:
-                    self.tables[t2.table] = t2
-                    self.aliases[t2.table] = self.aliases[t]
-        self.binary = None
-    def get_alias(self, table):
-        return self.aliases[table]
-    def visit_compound(self, compound):
-        self.visit_clauselist(compound)
-    def visit_clauselist(self, clist):
-        for i in range(0, len(clist.clauses)):
-            if isinstance(clist.clauses[i], schema.Column) and self.tables.has_key(clist.clauses[i].table):
-                orig = clist.clauses[i]
-                clist.clauses[i] = self.get_alias(clist.clauses[i].table).corresponding_column(clist.clauses[i])
-    def visit_binary(self, binary):
-        if isinstance(binary.left, schema.Column) and self.tables.has_key(binary.left.table):
-            binary.left = self.get_alias(binary.left.table).corresponding_column(binary.left)
-        if isinstance(binary.right, schema.Column) and self.tables.has_key(binary.right.table):
-            binary.right = self.get_alias(binary.right.table).corresponding_column(binary.right)
 
 class BinaryVisitor(sql.ClauseVisitor):
     def __init__(self, func):
