@@ -4,13 +4,13 @@
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
 
-import sets
+import sqlalchemy.util as util
 import sqlalchemy.sql as sql
 
 class CascadeOptions(object):
     """keeps track of the options sent to relation().cascade"""
     def __init__(self, arg=""):
-        values = sets.Set([c.strip() for c in arg.split(',')])
+        values = util.Set([c.strip() for c in arg.split(',')])
         self.delete_orphan = "delete-orphan" in values
         self.delete = "delete" in values or self.delete_orphan or "all" in values
         self.save_update = "save-update" in values or "all" in values
@@ -22,9 +22,9 @@ class CascadeOptions(object):
     
 
 def polymorphic_union(table_map, typecolname, aliasname='p_union'):
-    colnames = sets.Set()
+    colnames = util.Set()
     colnamemaps = {}
-    
+    types = {}
     for key in table_map.keys():
         table = table_map[key]
 
@@ -37,13 +37,14 @@ def polymorphic_union(table_map, typecolname, aliasname='p_union'):
         for c in table.c:
             colnames.add(c.name)
             m[c.name] = c
+            types[c.name] = c.type
         colnamemaps[table] = m
         
     def col(name, table):
         try:
             return colnamemaps[table][name]
         except KeyError:
-            return sql.null().label(name)
+            return sql.cast(sql.null(), types[name]).label(name)
 
     result = []
     for type, table in table_map.iteritems():
