@@ -125,10 +125,15 @@ def not_(clause):
     clause.parens=True
     return BooleanExpression(TextClause("NOT"), clause, None)
 
-def between_(ctest, cleft, cright):
-    """ returns BETWEEN predicate clause (clausetest BETWEEN clauseleft AND clauseright) """
-    return BooleanExpression(ctest, and_(cleft, cright), 'BETWEEN')
-between = between_
+def between(ctest, cleft, cright):
+    """ returns BETWEEN predicate clause (clausetest BETWEEN clauseleft AND clauseright).
+    
+    this is better called off a ColumnElement directly, i.e.
+    
+    column.between(value1, value2). 
+    """
+    return BooleanExpression(ctest, and_(_check_literal(cleft, ctest.type), _check_literal(cright, ctest.type)), 'BETWEEN')
+between_ = between
 
 def case(whens, value=None, else_=None):
     """ SQL CASE statement -- whens are a sequence of pairs to be translated into "when / then" clauses;
@@ -148,12 +153,9 @@ def cast(clause, totype, **kwargs):
          or
         cast(table.c.timestamp, DATE)
     """
-    # handle non-column clauses (e.g. cast(1234, TEXT)
-    if not hasattr(clause, 'label'):
-        clause = literal(clause)
-    totype = sqltypes.to_instance(totype)
-    return Function('CAST', CompoundClause("AS", clause, TypeClause(totype)), type=totype, **kwargs)
-        
+    return Cast(clause, totype, **kwargs)
+
+
 def exists(*args, **params):
     params['correlate'] = True
     s = select(*args, **params)
@@ -168,6 +170,12 @@ def union_all(*selects, **params):
 def alias(*args, **params):
     return Alias(*args, **params)
 
+def _check_literal(value, type):
+    if _is_literal(value):
+        return literal(value, type)
+    else:
+        return value
+
 def literal(value, type=None):
     """returns a literal clause, bound to a bind parameter.  
     
@@ -309,6 +317,7 @@ class ClauseVisitor(object):
     def visit_clauselist(self, list):pass
     def visit_calculatedclause(self, calcclause):pass
     def visit_function(self, func):pass
+    def visit_cast(self, cast):pass
     def visit_label(self, label):pass
     def visit_typeclause(self, typeclause):pass
             
@@ -551,7 +560,7 @@ class CompareMixin(object):
     def distinct(self):
         return CompoundClause(None,"DISTINCT", self)
     def between(self, cleft, cright):
-        return between_(self, cleft, cright)
+        return BooleanExpression(self, and_(self._check_literal(cleft), self._check_literal(cright)), 'BETWEEN')
     def op(self, operator):
         return lambda other: self._compare(operator, other)
     # and here come the math operators:
@@ -569,6 +578,11 @@ class CompareMixin(object):
         return self._operate('/', other)
     def _bind_param(self, obj):
         return BindParamClause('literal', obj, shortname=None, type=self.type)
+    def _check_literal(self, other):
+        if _is_literal(other):
+            return self._bind_param(other)
+        else:
+            return other
     def _compare(self, operator, obj):
         if obj is None or isinstance(obj, Null):
             if operator == '=':
@@ -577,8 +591,8 @@ class CompareMixin(object):
                 return BooleanExpression(self._compare_self(), null(), 'IS NOT')
             else:
                 raise exceptions.ArgumentError("Only '='/'!=' operators can be used with NULL")
-        elif _is_literal(obj):
-            obj = self._bind_param(obj)
+        else:
+            obj = self._check_literal(obj)
 
         return BooleanExpression(self._compare_self(), obj, operator, type=self._compare_type(obj))
     def _operate(self, operator, obj):
@@ -624,7 +638,7 @@ class ColumnElement(Selectable, CompareMixin):
         try:
             return self.__orig_set
         except AttributeError:
-            self.__orig_set = sets.Set([self])
+            self.__orig_set = util.Set([self])
             return self.__orig_set
     def _set_orig_set(self, s):
         if len(s) == 0:
@@ -809,7 +823,7 @@ class TextClause(ClauseElement):
         if bindparams is not None:
             for b in bindparams:
                 self.bindparams[b.key] = b
-            
+    columns = property(lambda s:[])        
     def accept_visitor(self, visitor): 
         for item in self.bindparams.values():
             item.accept_visitor(visitor)
@@ -958,7 +972,20 @@ class Function(CalculatedClause):
             c.accept_visitor(visitor)
         visitor.visit_function(self)
 
-
+class Cast(ColumnElement):
+    def __init__(self, clause, totype, **kwargs):
+        if not hasattr(clause, 'label'):
+            clause = literal(clause)
+        self.type = sqltypes.to_instance(totype)
+        self.clause = clause
+        self.typeclause = TypeClause(self.type)
+    def accept_visitor(self, visitor):
+        self.clause.accept_visitor(visitor)
+        self.typeclause.accept_visitor(visitor)
+        visitor.visit_cast(self)
+    def _get_from_objects(self):
+        return self.clause._get_from_objects()
+        
 class FunctionGenerator(object):
     """generates Function objects based on getattr calls"""
     def __init__(self, engine=None):
@@ -1334,7 +1361,7 @@ class CompoundSelect(SelectBaseMixin, FromClause):
         try:
             colset = self._col_map[col.name]
         except KeyError:
-            colset = sets.Set()
+            colset = util.Set()
             self._col_map[col.name] = colset
         [colset.add(c) for c in col.orig_set]
         col.orig_set = colset
@@ -1473,7 +1500,7 @@ class Select(SelectBaseMixin, FromClause):
         
     def append_from(self, fromclause):
         if type(fromclause) == str:
-            fromclause = FromClause(from_name = fromclause)
+            fromclause = TextClause(fromclause)
         fromclause.accept_visitor(self._correlator)
         fromclause._process_from_dict(self._froms, True)
     def _locate_oid_column(self):
