@@ -795,7 +795,25 @@ class EagerTest(MapperSuperTest):
         l = q.select((Item.c.item_name=='item 2') | (Item.c.item_name=='item 5') | (Item.c.item_name=='item 3'), order_by=[Item.c.item_id], limit=2)        
         self.assert_result(l, Item, *[item_keyword_result[1], item_keyword_result[2]])
         
+    def testmorelimit(self):
+        """test that the ORDER BY is propigated from the inner select to the outer select, when using the 
+        'wrapped' select statement resulting from the combination of eager loading and limit/offset clauses."""
+        ordermapper = mapper(Order, orders, properties = dict(
+                items = relation(mapper(Item, orderitems), lazy = False)
+            ))
+
+        m = mapper(User, users, properties = dict(
+            addresses = relation(mapper(Address, addresses), lazy = False),
+            orders = relation(ordermapper, primaryjoin = users.c.user_id==orders.c.user_id, lazy = False),
+        ))
+        sess = create_session()
+        q = sess.query(m)
         
+        l = q.select(q.join_to('orders'), order_by=desc(orders.c.user_id), limit=2, offset=1)
+        self.assert_result(l, User, *(user_all_result[2], user_all_result[0]))
+        
+        l = q.select(q.join_to('addresses'), order_by=desc(addresses.c.email_address), limit=1, offset=0)
+        self.assert_result(l, User, *(user_all_result[0],))
         
     def testonetoone(self):
         m = mapper(User, users, properties = dict(
